@@ -1,6 +1,7 @@
 /* 
  * Copyright © 2008-2011 Kristian Høgsberg
  * Copyright © 2010-2011 Intel Corporation
+ * Copyright © 2012-2013 Collabora, Ltd.
  * 
  * Permission to use, copy, modify, distribute, and sell this
  * software and its documentation for any purpose is hereby granted
@@ -58,6 +59,8 @@ struct wl_keyboard;
 struct wl_touch;
 struct wl_output;
 struct wl_region;
+struct wl_subcompositor;
+struct wl_subsurface;
 
 extern const struct wl_interface wl_display_interface;
 extern const struct wl_interface wl_registry_interface;
@@ -79,6 +82,8 @@ extern const struct wl_interface wl_keyboard_interface;
 extern const struct wl_interface wl_touch_interface;
 extern const struct wl_interface wl_output_interface;
 extern const struct wl_interface wl_region_interface;
+extern const struct wl_interface wl_subcompositor_interface;
+extern const struct wl_interface wl_subsurface_interface;
 
 #ifndef WL_DISPLAY_ERROR_ENUM
 #define WL_DISPLAY_ERROR_ENUM
@@ -170,13 +175,8 @@ wl_display_sync(struct wl_display *wl_display)
 {
 	struct wl_proxy *callback;
 
-	callback = wl_proxy_create((struct wl_proxy *) wl_display,
-			     &wl_callback_interface);
-	if (!callback)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_display,
-			 WL_DISPLAY_SYNC, callback);
+	callback = wl_proxy_marshal_constructor((struct wl_proxy *) wl_display,
+			 WL_DISPLAY_SYNC, &wl_callback_interface, NULL);
 
 	return (struct wl_callback *) callback;
 }
@@ -186,13 +186,8 @@ wl_display_get_registry(struct wl_display *wl_display)
 {
 	struct wl_proxy *callback;
 
-	callback = wl_proxy_create((struct wl_proxy *) wl_display,
-			     &wl_registry_interface);
-	if (!callback)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_display,
-			 WL_DISPLAY_GET_REGISTRY, callback);
+	callback = wl_proxy_marshal_constructor((struct wl_proxy *) wl_display,
+			 WL_DISPLAY_GET_REGISTRY, &wl_registry_interface, NULL);
 
 	return (struct wl_registry *) callback;
 }
@@ -290,13 +285,8 @@ wl_registry_bind(struct wl_registry *wl_registry, uint32_t name, const struct wl
 {
 	struct wl_proxy *id;
 
-	id = wl_proxy_create((struct wl_proxy *) wl_registry,
-			     interface);
-	if (!id)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_registry,
-			 WL_REGISTRY_BIND, name, interface->name, version, id);
+	id = wl_proxy_marshal_constructor((struct wl_proxy *) wl_registry,
+			 WL_REGISTRY_BIND, interface, name, interface->name, version, NULL);
 
 	return (void *) id;
 }
@@ -372,13 +362,8 @@ wl_compositor_create_surface(struct wl_compositor *wl_compositor)
 {
 	struct wl_proxy *id;
 
-	id = wl_proxy_create((struct wl_proxy *) wl_compositor,
-			     &wl_surface_interface);
-	if (!id)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_compositor,
-			 WL_COMPOSITOR_CREATE_SURFACE, id);
+	id = wl_proxy_marshal_constructor((struct wl_proxy *) wl_compositor,
+			 WL_COMPOSITOR_CREATE_SURFACE, &wl_surface_interface, NULL);
 
 	return (struct wl_surface *) id;
 }
@@ -388,13 +373,8 @@ wl_compositor_create_region(struct wl_compositor *wl_compositor)
 {
 	struct wl_proxy *id;
 
-	id = wl_proxy_create((struct wl_proxy *) wl_compositor,
-			     &wl_region_interface);
-	if (!id)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_compositor,
-			 WL_COMPOSITOR_CREATE_REGION, id);
+	id = wl_proxy_marshal_constructor((struct wl_proxy *) wl_compositor,
+			 WL_COMPOSITOR_CREATE_REGION, &wl_region_interface, NULL);
 
 	return (struct wl_region *) id;
 }
@@ -420,13 +400,8 @@ wl_shm_pool_create_buffer(struct wl_shm_pool *wl_shm_pool, int32_t offset, int32
 {
 	struct wl_proxy *id;
 
-	id = wl_proxy_create((struct wl_proxy *) wl_shm_pool,
-			     &wl_buffer_interface);
-	if (!id)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_shm_pool,
-			 WL_SHM_POOL_CREATE_BUFFER, id, offset, width, height, stride, format);
+	id = wl_proxy_marshal_constructor((struct wl_proxy *) wl_shm_pool,
+			 WL_SHM_POOL_CREATE_BUFFER, &wl_buffer_interface, NULL, offset, width, height, stride, format);
 
 	return (struct wl_buffer *) id;
 }
@@ -471,12 +446,127 @@ enum wl_shm_error {
  * wl_shm_format - pixel formats
  * @WL_SHM_FORMAT_ARGB8888: 32-bit ARGB format
  * @WL_SHM_FORMAT_XRGB8888: 32-bit RGB format
+ * @WL_SHM_FORMAT_C8: (none)
+ * @WL_SHM_FORMAT_RGB332: (none)
+ * @WL_SHM_FORMAT_BGR233: (none)
+ * @WL_SHM_FORMAT_XRGB4444: (none)
+ * @WL_SHM_FORMAT_XBGR4444: (none)
+ * @WL_SHM_FORMAT_RGBX4444: (none)
+ * @WL_SHM_FORMAT_BGRX4444: (none)
+ * @WL_SHM_FORMAT_ARGB4444: (none)
+ * @WL_SHM_FORMAT_ABGR4444: (none)
+ * @WL_SHM_FORMAT_RGBA4444: (none)
+ * @WL_SHM_FORMAT_BGRA4444: (none)
+ * @WL_SHM_FORMAT_XRGB1555: (none)
+ * @WL_SHM_FORMAT_XBGR1555: (none)
+ * @WL_SHM_FORMAT_RGBX5551: (none)
+ * @WL_SHM_FORMAT_BGRX5551: (none)
+ * @WL_SHM_FORMAT_ARGB1555: (none)
+ * @WL_SHM_FORMAT_ABGR1555: (none)
+ * @WL_SHM_FORMAT_RGBA5551: (none)
+ * @WL_SHM_FORMAT_BGRA5551: (none)
+ * @WL_SHM_FORMAT_RGB565: (none)
+ * @WL_SHM_FORMAT_BGR565: (none)
+ * @WL_SHM_FORMAT_RGB888: (none)
+ * @WL_SHM_FORMAT_BGR888: (none)
+ * @WL_SHM_FORMAT_XBGR8888: (none)
+ * @WL_SHM_FORMAT_RGBX8888: (none)
+ * @WL_SHM_FORMAT_BGRX8888: (none)
+ * @WL_SHM_FORMAT_ABGR8888: (none)
+ * @WL_SHM_FORMAT_RGBA8888: (none)
+ * @WL_SHM_FORMAT_BGRA8888: (none)
+ * @WL_SHM_FORMAT_XRGB2101010: (none)
+ * @WL_SHM_FORMAT_XBGR2101010: (none)
+ * @WL_SHM_FORMAT_RGBX1010102: (none)
+ * @WL_SHM_FORMAT_BGRX1010102: (none)
+ * @WL_SHM_FORMAT_ARGB2101010: (none)
+ * @WL_SHM_FORMAT_ABGR2101010: (none)
+ * @WL_SHM_FORMAT_RGBA1010102: (none)
+ * @WL_SHM_FORMAT_BGRA1010102: (none)
+ * @WL_SHM_FORMAT_YUYV: (none)
+ * @WL_SHM_FORMAT_YVYU: (none)
+ * @WL_SHM_FORMAT_UYVY: (none)
+ * @WL_SHM_FORMAT_VYUY: (none)
+ * @WL_SHM_FORMAT_AYUV: (none)
+ * @WL_SHM_FORMAT_NV12: (none)
+ * @WL_SHM_FORMAT_NV21: (none)
+ * @WL_SHM_FORMAT_NV16: (none)
+ * @WL_SHM_FORMAT_NV61: (none)
+ * @WL_SHM_FORMAT_YUV410: (none)
+ * @WL_SHM_FORMAT_YVU410: (none)
+ * @WL_SHM_FORMAT_YUV411: (none)
+ * @WL_SHM_FORMAT_YVU411: (none)
+ * @WL_SHM_FORMAT_YUV420: (none)
+ * @WL_SHM_FORMAT_YVU420: (none)
+ * @WL_SHM_FORMAT_YUV422: (none)
+ * @WL_SHM_FORMAT_YVU422: (none)
+ * @WL_SHM_FORMAT_YUV444: (none)
+ * @WL_SHM_FORMAT_YVU444: (none)
  *
  * This describes the memory layout of an individual pixel.
+ *
+ * All renderers should support argb8888 and xrgb8888 but any other formats
+ * are optional and may not be supported by the particular renderer in use.
  */
 enum wl_shm_format {
 	WL_SHM_FORMAT_ARGB8888 = 0,
 	WL_SHM_FORMAT_XRGB8888 = 1,
+	WL_SHM_FORMAT_C8 = 0x20203843,
+	WL_SHM_FORMAT_RGB332 = 0x38424752,
+	WL_SHM_FORMAT_BGR233 = 0x38524742,
+	WL_SHM_FORMAT_XRGB4444 = 0x32315258,
+	WL_SHM_FORMAT_XBGR4444 = 0x32314258,
+	WL_SHM_FORMAT_RGBX4444 = 0x32315852,
+	WL_SHM_FORMAT_BGRX4444 = 0x32315842,
+	WL_SHM_FORMAT_ARGB4444 = 0x32315241,
+	WL_SHM_FORMAT_ABGR4444 = 0x32314241,
+	WL_SHM_FORMAT_RGBA4444 = 0x32314152,
+	WL_SHM_FORMAT_BGRA4444 = 0x32314142,
+	WL_SHM_FORMAT_XRGB1555 = 0x35315258,
+	WL_SHM_FORMAT_XBGR1555 = 0x35314258,
+	WL_SHM_FORMAT_RGBX5551 = 0x35315852,
+	WL_SHM_FORMAT_BGRX5551 = 0x35315842,
+	WL_SHM_FORMAT_ARGB1555 = 0x35315241,
+	WL_SHM_FORMAT_ABGR1555 = 0x35314241,
+	WL_SHM_FORMAT_RGBA5551 = 0x35314152,
+	WL_SHM_FORMAT_BGRA5551 = 0x35314142,
+	WL_SHM_FORMAT_RGB565 = 0x36314752,
+	WL_SHM_FORMAT_BGR565 = 0x36314742,
+	WL_SHM_FORMAT_RGB888 = 0x34324752,
+	WL_SHM_FORMAT_BGR888 = 0x34324742,
+	WL_SHM_FORMAT_XBGR8888 = 0x34324258,
+	WL_SHM_FORMAT_RGBX8888 = 0x34325852,
+	WL_SHM_FORMAT_BGRX8888 = 0x34325842,
+	WL_SHM_FORMAT_ABGR8888 = 0x34324241,
+	WL_SHM_FORMAT_RGBA8888 = 0x34324152,
+	WL_SHM_FORMAT_BGRA8888 = 0x34324142,
+	WL_SHM_FORMAT_XRGB2101010 = 0x30335258,
+	WL_SHM_FORMAT_XBGR2101010 = 0x30334258,
+	WL_SHM_FORMAT_RGBX1010102 = 0x30335852,
+	WL_SHM_FORMAT_BGRX1010102 = 0x30335842,
+	WL_SHM_FORMAT_ARGB2101010 = 0x30335241,
+	WL_SHM_FORMAT_ABGR2101010 = 0x30334241,
+	WL_SHM_FORMAT_RGBA1010102 = 0x30334152,
+	WL_SHM_FORMAT_BGRA1010102 = 0x30334142,
+	WL_SHM_FORMAT_YUYV = 0x56595559,
+	WL_SHM_FORMAT_YVYU = 0x55595659,
+	WL_SHM_FORMAT_UYVY = 0x59565955,
+	WL_SHM_FORMAT_VYUY = 0x59555956,
+	WL_SHM_FORMAT_AYUV = 0x56555941,
+	WL_SHM_FORMAT_NV12 = 0x3231564e,
+	WL_SHM_FORMAT_NV21 = 0x3132564e,
+	WL_SHM_FORMAT_NV16 = 0x3631564e,
+	WL_SHM_FORMAT_NV61 = 0x3136564e,
+	WL_SHM_FORMAT_YUV410 = 0x39565559,
+	WL_SHM_FORMAT_YVU410 = 0x39555659,
+	WL_SHM_FORMAT_YUV411 = 0x31315559,
+	WL_SHM_FORMAT_YVU411 = 0x31315659,
+	WL_SHM_FORMAT_YUV420 = 0x32315559,
+	WL_SHM_FORMAT_YVU420 = 0x32315659,
+	WL_SHM_FORMAT_YUV422 = 0x36315559,
+	WL_SHM_FORMAT_YVU422 = 0x36315659,
+	WL_SHM_FORMAT_YUV444 = 0x34325559,
+	WL_SHM_FORMAT_YVU444 = 0x34325659,
 };
 #endif /* WL_SHM_FORMAT_ENUM */
 
@@ -538,13 +628,8 @@ wl_shm_create_pool(struct wl_shm *wl_shm, int32_t fd, int32_t size)
 {
 	struct wl_proxy *id;
 
-	id = wl_proxy_create((struct wl_proxy *) wl_shm,
-			     &wl_shm_pool_interface);
-	if (!id)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_shm,
-			 WL_SHM_CREATE_POOL, id, fd, size);
+	id = wl_proxy_marshal_constructor((struct wl_proxy *) wl_shm,
+			 WL_SHM_CREATE_POOL, &wl_shm_pool_interface, NULL, fd, size);
 
 	return (struct wl_shm_pool *) id;
 }
@@ -939,13 +1024,8 @@ wl_data_device_manager_create_data_source(struct wl_data_device_manager *wl_data
 {
 	struct wl_proxy *id;
 
-	id = wl_proxy_create((struct wl_proxy *) wl_data_device_manager,
-			     &wl_data_source_interface);
-	if (!id)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_data_device_manager,
-			 WL_DATA_DEVICE_MANAGER_CREATE_DATA_SOURCE, id);
+	id = wl_proxy_marshal_constructor((struct wl_proxy *) wl_data_device_manager,
+			 WL_DATA_DEVICE_MANAGER_CREATE_DATA_SOURCE, &wl_data_source_interface, NULL);
 
 	return (struct wl_data_source *) id;
 }
@@ -955,13 +1035,8 @@ wl_data_device_manager_get_data_device(struct wl_data_device_manager *wl_data_de
 {
 	struct wl_proxy *id;
 
-	id = wl_proxy_create((struct wl_proxy *) wl_data_device_manager,
-			     &wl_data_device_interface);
-	if (!id)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_data_device_manager,
-			 WL_DATA_DEVICE_MANAGER_GET_DATA_DEVICE, id, seat);
+	id = wl_proxy_marshal_constructor((struct wl_proxy *) wl_data_device_manager,
+			 WL_DATA_DEVICE_MANAGER_GET_DATA_DEVICE, &wl_data_device_interface, NULL, seat);
 
 	return (struct wl_data_device *) id;
 }
@@ -991,13 +1066,8 @@ wl_shell_get_shell_surface(struct wl_shell *wl_shell, struct wl_surface *surface
 {
 	struct wl_proxy *id;
 
-	id = wl_proxy_create((struct wl_proxy *) wl_shell,
-			     &wl_shell_surface_interface);
-	if (!id)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_shell,
-			 WL_SHELL_GET_SHELL_SURFACE, id, surface);
+	id = wl_proxy_marshal_constructor((struct wl_proxy *) wl_shell,
+			 WL_SHELL_GET_SHELL_SURFACE, &wl_shell_surface_interface, NULL, surface);
 
 	return (struct wl_shell_surface *) id;
 }
@@ -1352,13 +1422,8 @@ wl_surface_frame(struct wl_surface *wl_surface)
 {
 	struct wl_proxy *callback;
 
-	callback = wl_proxy_create((struct wl_proxy *) wl_surface,
-			     &wl_callback_interface);
-	if (!callback)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_surface,
-			 WL_SURFACE_FRAME, callback);
+	callback = wl_proxy_marshal_constructor((struct wl_proxy *) wl_surface,
+			 WL_SURFACE_FRAME, &wl_callback_interface, NULL);
 
 	return (struct wl_callback *) callback;
 }
@@ -1487,13 +1552,8 @@ wl_seat_get_pointer(struct wl_seat *wl_seat)
 {
 	struct wl_proxy *id;
 
-	id = wl_proxy_create((struct wl_proxy *) wl_seat,
-			     &wl_pointer_interface);
-	if (!id)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_seat,
-			 WL_SEAT_GET_POINTER, id);
+	id = wl_proxy_marshal_constructor((struct wl_proxy *) wl_seat,
+			 WL_SEAT_GET_POINTER, &wl_pointer_interface, NULL);
 
 	return (struct wl_pointer *) id;
 }
@@ -1503,13 +1563,8 @@ wl_seat_get_keyboard(struct wl_seat *wl_seat)
 {
 	struct wl_proxy *id;
 
-	id = wl_proxy_create((struct wl_proxy *) wl_seat,
-			     &wl_keyboard_interface);
-	if (!id)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_seat,
-			 WL_SEAT_GET_KEYBOARD, id);
+	id = wl_proxy_marshal_constructor((struct wl_proxy *) wl_seat,
+			 WL_SEAT_GET_KEYBOARD, &wl_keyboard_interface, NULL);
 
 	return (struct wl_keyboard *) id;
 }
@@ -1519,13 +1574,8 @@ wl_seat_get_touch(struct wl_seat *wl_seat)
 {
 	struct wl_proxy *id;
 
-	id = wl_proxy_create((struct wl_proxy *) wl_seat,
-			     &wl_touch_interface);
-	if (!id)
-		return NULL;
-
-	wl_proxy_marshal((struct wl_proxy *) wl_seat,
-			 WL_SEAT_GET_TOUCH, id);
+	id = wl_proxy_marshal_constructor((struct wl_proxy *) wl_seat,
+			 WL_SEAT_GET_TOUCH, &wl_touch_interface, NULL);
 
 	return (struct wl_touch *) id;
 }
@@ -1590,6 +1640,7 @@ struct wl_pointer_listener {
 	 * When an seat's focus enters a surface, the pointer image is
 	 * undefined and a client should respond to this event by setting
 	 * an appropriate pointer image with the set_cursor request.
+	 * @since: 3
 	 */
 	void (*enter)(void *data,
 		      struct wl_pointer *wl_pointer,
@@ -1607,6 +1658,7 @@ struct wl_pointer_listener {
 	 *
 	 * The leave notification is sent before the enter notification for
 	 * the new focus.
+	 * @since: 3
 	 */
 	void (*leave)(void *data,
 		      struct wl_pointer *wl_pointer,
@@ -1621,6 +1673,7 @@ struct wl_pointer_listener {
 	 * Notification of pointer location change. The arguments
 	 * surface_x and surface_y are the location relative to the focused
 	 * surface.
+	 * @since: 3
 	 */
 	void (*motion)(void *data,
 		       struct wl_pointer *wl_pointer,
@@ -1639,6 +1692,7 @@ struct wl_pointer_listener {
 	 * The location of the click is given by the last motion or enter
 	 * event. The time argument is a timestamp with millisecond
 	 * granularity, with an undefined base.
+	 * @since: 3
 	 */
 	void (*button)(void *data,
 		       struct wl_pointer *wl_pointer,
@@ -1668,6 +1722,7 @@ struct wl_pointer_listener {
 	 *
 	 * When applicable, clients can transform its view relative to the
 	 * scroll distance.
+	 * @since: 3
 	 */
 	void (*axis)(void *data,
 		     struct wl_pointer *wl_pointer,
@@ -1685,6 +1740,7 @@ wl_pointer_add_listener(struct wl_pointer *wl_pointer,
 }
 
 #define WL_POINTER_SET_CURSOR	0
+#define WL_POINTER_RELEASE	1
 
 static inline void
 wl_pointer_set_user_data(struct wl_pointer *wl_pointer, void *user_data)
@@ -1711,6 +1767,15 @@ wl_pointer_set_cursor(struct wl_pointer *wl_pointer, uint32_t serial, struct wl_
 			 WL_POINTER_SET_CURSOR, serial, surface, hotspot_x, hotspot_y);
 }
 
+static inline void
+wl_pointer_release(struct wl_pointer *wl_pointer)
+{
+	wl_proxy_marshal((struct wl_proxy *) wl_pointer,
+			 WL_POINTER_RELEASE);
+
+	wl_proxy_destroy((struct wl_proxy *) wl_pointer);
+}
+
 #ifndef WL_KEYBOARD_KEYMAP_FORMAT_ENUM
 #define WL_KEYBOARD_KEYMAP_FORMAT_ENUM
 /**
@@ -1763,6 +1828,7 @@ struct wl_keyboard_listener {
 	 *
 	 * This event provides a file descriptor to the client which can
 	 * be memory-mapped to provide a keyboard mapping description.
+	 * @since: 3
 	 */
 	void (*keymap)(void *data,
 		       struct wl_keyboard *wl_keyboard,
@@ -1777,6 +1843,7 @@ struct wl_keyboard_listener {
 	 *
 	 * Notification that this seat's keyboard focus is on a certain
 	 * surface.
+	 * @since: 3
 	 */
 	void (*enter)(void *data,
 		      struct wl_keyboard *wl_keyboard,
@@ -1793,6 +1860,7 @@ struct wl_keyboard_listener {
 	 *
 	 * The leave notification is sent before the enter notification for
 	 * the new focus.
+	 * @since: 3
 	 */
 	void (*leave)(void *data,
 		      struct wl_keyboard *wl_keyboard,
@@ -1807,6 +1875,7 @@ struct wl_keyboard_listener {
 	 *
 	 * A key was pressed or released. The time argument is a
 	 * timestamp with millisecond granularity, with an undefined base.
+	 * @since: 3
 	 */
 	void (*key)(void *data,
 		    struct wl_keyboard *wl_keyboard,
@@ -1824,6 +1893,7 @@ struct wl_keyboard_listener {
 	 *
 	 * Notifies clients that the modifier and/or group state has
 	 * changed, and it should update its local state.
+	 * @since: 3
 	 */
 	void (*modifiers)(void *data,
 			  struct wl_keyboard *wl_keyboard,
@@ -1842,6 +1912,8 @@ wl_keyboard_add_listener(struct wl_keyboard *wl_keyboard,
 				     (void (**)(void)) listener, data);
 }
 
+#define WL_KEYBOARD_RELEASE	0
+
 static inline void
 wl_keyboard_set_user_data(struct wl_keyboard *wl_keyboard, void *user_data)
 {
@@ -1860,6 +1932,15 @@ wl_keyboard_destroy(struct wl_keyboard *wl_keyboard)
 	wl_proxy_destroy((struct wl_proxy *) wl_keyboard);
 }
 
+static inline void
+wl_keyboard_release(struct wl_keyboard *wl_keyboard)
+{
+	wl_proxy_marshal((struct wl_proxy *) wl_keyboard,
+			 WL_KEYBOARD_RELEASE);
+
+	wl_proxy_destroy((struct wl_proxy *) wl_keyboard);
+}
+
 /**
  * wl_touch - touchscreen input device
  * @down: touch down event and beginning of a touch sequence
@@ -1891,6 +1972,7 @@ struct wl_touch_listener {
 	 * point is assigned a unique @id. Future events from this
 	 * touchpoint reference this ID. The ID ceases to be valid after a
 	 * touch up event and may be re-used in the future.
+	 * @since: 3
 	 */
 	void (*down)(void *data,
 		     struct wl_touch *wl_touch,
@@ -1909,6 +1991,7 @@ struct wl_touch_listener {
 	 * The touch point has disappeared. No further events will be
 	 * sent for this touchpoint and the touch point's ID is released
 	 * and may be re-used in a future touch down event.
+	 * @since: 3
 	 */
 	void (*up)(void *data,
 		   struct wl_touch *wl_touch,
@@ -1923,6 +2006,7 @@ struct wl_touch_listener {
 	 * @y: y coordinate in surface-relative coordinates
 	 *
 	 * A touchpoint has changed coordinates.
+	 * @since: 3
 	 */
 	void (*motion)(void *data,
 		       struct wl_touch *wl_touch,
@@ -1934,6 +2018,7 @@ struct wl_touch_listener {
 	 * frame - end of touch frame event
 	 *
 	 * Indicates the end of a contact point list.
+	 * @since: 3
 	 */
 	void (*frame)(void *data,
 		      struct wl_touch *wl_touch);
@@ -1946,6 +2031,7 @@ struct wl_touch_listener {
 	 * points currently active on this client's surface. The client is
 	 * responsible for finalizing the touch points, future touch points
 	 * on this surface may re-use the touch point ID.
+	 * @since: 3
 	 */
 	void (*cancel)(void *data,
 		       struct wl_touch *wl_touch);
@@ -1959,6 +2045,8 @@ wl_touch_add_listener(struct wl_touch *wl_touch,
 				     (void (**)(void)) listener, data);
 }
 
+#define WL_TOUCH_RELEASE	0
+
 static inline void
 wl_touch_set_user_data(struct wl_touch *wl_touch, void *user_data)
 {
@@ -1977,6 +2065,15 @@ wl_touch_destroy(struct wl_touch *wl_touch)
 	wl_proxy_destroy((struct wl_proxy *) wl_touch);
 }
 
+static inline void
+wl_touch_release(struct wl_touch *wl_touch)
+{
+	wl_proxy_marshal((struct wl_proxy *) wl_touch,
+			 WL_TOUCH_RELEASE);
+
+	wl_proxy_destroy((struct wl_proxy *) wl_touch);
+}
+
 #ifndef WL_OUTPUT_SUBPIXEL_ENUM
 #define WL_OUTPUT_SUBPIXEL_ENUM
 /**
@@ -2223,6 +2320,118 @@ wl_region_subtract(struct wl_region *wl_region, int32_t x, int32_t y, int32_t wi
 			 WL_REGION_SUBTRACT, x, y, width, height);
 }
 
+#ifndef WL_SUBCOMPOSITOR_ERROR_ENUM
+#define WL_SUBCOMPOSITOR_ERROR_ENUM
+enum wl_subcompositor_error {
+	WL_SUBCOMPOSITOR_ERROR_BAD_SURFACE = 0,
+};
+#endif /* WL_SUBCOMPOSITOR_ERROR_ENUM */
+
+#define WL_SUBCOMPOSITOR_DESTROY	0
+#define WL_SUBCOMPOSITOR_GET_SUBSURFACE	1
+
+static inline void
+wl_subcompositor_set_user_data(struct wl_subcompositor *wl_subcompositor, void *user_data)
+{
+	wl_proxy_set_user_data((struct wl_proxy *) wl_subcompositor, user_data);
+}
+
+static inline void *
+wl_subcompositor_get_user_data(struct wl_subcompositor *wl_subcompositor)
+{
+	return wl_proxy_get_user_data((struct wl_proxy *) wl_subcompositor);
+}
+
+static inline void
+wl_subcompositor_destroy(struct wl_subcompositor *wl_subcompositor)
+{
+	wl_proxy_marshal((struct wl_proxy *) wl_subcompositor,
+			 WL_SUBCOMPOSITOR_DESTROY);
+
+	wl_proxy_destroy((struct wl_proxy *) wl_subcompositor);
+}
+
+static inline struct wl_subsurface *
+wl_subcompositor_get_subsurface(struct wl_subcompositor *wl_subcompositor, struct wl_surface *surface, struct wl_surface *parent)
+{
+	struct wl_proxy *id;
+
+	id = wl_proxy_marshal_constructor((struct wl_proxy *) wl_subcompositor,
+			 WL_SUBCOMPOSITOR_GET_SUBSURFACE, &wl_subsurface_interface, NULL, surface, parent);
+
+	return (struct wl_subsurface *) id;
+}
+
+#ifndef WL_SUBSURFACE_ERROR_ENUM
+#define WL_SUBSURFACE_ERROR_ENUM
+enum wl_subsurface_error {
+	WL_SUBSURFACE_ERROR_BAD_SURFACE = 0,
+};
+#endif /* WL_SUBSURFACE_ERROR_ENUM */
+
+#define WL_SUBSURFACE_DESTROY	0
+#define WL_SUBSURFACE_SET_POSITION	1
+#define WL_SUBSURFACE_PLACE_ABOVE	2
+#define WL_SUBSURFACE_PLACE_BELOW	3
+#define WL_SUBSURFACE_SET_SYNC	4
+#define WL_SUBSURFACE_SET_DESYNC	5
+
+static inline void
+wl_subsurface_set_user_data(struct wl_subsurface *wl_subsurface, void *user_data)
+{
+	wl_proxy_set_user_data((struct wl_proxy *) wl_subsurface, user_data);
+}
+
+static inline void *
+wl_subsurface_get_user_data(struct wl_subsurface *wl_subsurface)
+{
+	return wl_proxy_get_user_data((struct wl_proxy *) wl_subsurface);
+}
+
+static inline void
+wl_subsurface_destroy(struct wl_subsurface *wl_subsurface)
+{
+	wl_proxy_marshal((struct wl_proxy *) wl_subsurface,
+			 WL_SUBSURFACE_DESTROY);
+
+	wl_proxy_destroy((struct wl_proxy *) wl_subsurface);
+}
+
+static inline void
+wl_subsurface_set_position(struct wl_subsurface *wl_subsurface, int32_t x, int32_t y)
+{
+	wl_proxy_marshal((struct wl_proxy *) wl_subsurface,
+			 WL_SUBSURFACE_SET_POSITION, x, y);
+}
+
+static inline void
+wl_subsurface_place_above(struct wl_subsurface *wl_subsurface, struct wl_surface *sibling)
+{
+	wl_proxy_marshal((struct wl_proxy *) wl_subsurface,
+			 WL_SUBSURFACE_PLACE_ABOVE, sibling);
+}
+
+static inline void
+wl_subsurface_place_below(struct wl_subsurface *wl_subsurface, struct wl_surface *sibling)
+{
+	wl_proxy_marshal((struct wl_proxy *) wl_subsurface,
+			 WL_SUBSURFACE_PLACE_BELOW, sibling);
+}
+
+static inline void
+wl_subsurface_set_sync(struct wl_subsurface *wl_subsurface)
+{
+	wl_proxy_marshal((struct wl_proxy *) wl_subsurface,
+			 WL_SUBSURFACE_SET_SYNC);
+}
+
+static inline void
+wl_subsurface_set_desync(struct wl_subsurface *wl_subsurface)
+{
+	wl_proxy_marshal((struct wl_proxy *) wl_subsurface,
+			 WL_SUBSURFACE_SET_DESYNC);
+}
+
 #ifdef  __cplusplus
 }
 #endif
