@@ -1,6 +1,7 @@
 /* 
  * Copyright © 2008-2011 Kristian Høgsberg
  * Copyright © 2010-2011 Intel Corporation
+ * Copyright © 2012-2013 Collabora, Ltd.
  * 
  * Permission to use, copy, modify, distribute, and sell this
  * software and its documentation for any purpose is hereby granted
@@ -58,6 +59,8 @@ struct wl_keyboard;
 struct wl_touch;
 struct wl_output;
 struct wl_region;
+struct wl_subcompositor;
+struct wl_subsurface;
 
 extern const struct wl_interface wl_display_interface;
 extern const struct wl_interface wl_registry_interface;
@@ -79,6 +82,8 @@ extern const struct wl_interface wl_keyboard_interface;
 extern const struct wl_interface wl_touch_interface;
 extern const struct wl_interface wl_output_interface;
 extern const struct wl_interface wl_region_interface;
+extern const struct wl_interface wl_subcompositor_interface;
+extern const struct wl_interface wl_subsurface_interface;
 
 #ifndef WL_DISPLAY_ERROR_ENUM
 #define WL_DISPLAY_ERROR_ENUM
@@ -114,9 +119,9 @@ struct wl_display_interface {
 	 *
 	 * The sync request asks the server to emit the 'done' event on
 	 * the returned wl_callback object. Since requests are handled
-	 * in-order and events are delivered in-order, this can used as a
-	 * barrier to ensure all previous requests and the resulting events
-	 * have been handled.
+	 * in-order and events are delivered in-order, this can be used as
+	 * a barrier to ensure all previous requests and the resulting
+	 * events have been handled.
 	 *
 	 * The object returned by this request will be destroyed by the
 	 * compositor after the callback is fired and as such the client
@@ -322,12 +327,127 @@ enum wl_shm_error {
  * wl_shm_format - pixel formats
  * @WL_SHM_FORMAT_ARGB8888: 32-bit ARGB format
  * @WL_SHM_FORMAT_XRGB8888: 32-bit RGB format
+ * @WL_SHM_FORMAT_C8: (none)
+ * @WL_SHM_FORMAT_RGB332: (none)
+ * @WL_SHM_FORMAT_BGR233: (none)
+ * @WL_SHM_FORMAT_XRGB4444: (none)
+ * @WL_SHM_FORMAT_XBGR4444: (none)
+ * @WL_SHM_FORMAT_RGBX4444: (none)
+ * @WL_SHM_FORMAT_BGRX4444: (none)
+ * @WL_SHM_FORMAT_ARGB4444: (none)
+ * @WL_SHM_FORMAT_ABGR4444: (none)
+ * @WL_SHM_FORMAT_RGBA4444: (none)
+ * @WL_SHM_FORMAT_BGRA4444: (none)
+ * @WL_SHM_FORMAT_XRGB1555: (none)
+ * @WL_SHM_FORMAT_XBGR1555: (none)
+ * @WL_SHM_FORMAT_RGBX5551: (none)
+ * @WL_SHM_FORMAT_BGRX5551: (none)
+ * @WL_SHM_FORMAT_ARGB1555: (none)
+ * @WL_SHM_FORMAT_ABGR1555: (none)
+ * @WL_SHM_FORMAT_RGBA5551: (none)
+ * @WL_SHM_FORMAT_BGRA5551: (none)
+ * @WL_SHM_FORMAT_RGB565: (none)
+ * @WL_SHM_FORMAT_BGR565: (none)
+ * @WL_SHM_FORMAT_RGB888: (none)
+ * @WL_SHM_FORMAT_BGR888: (none)
+ * @WL_SHM_FORMAT_XBGR8888: (none)
+ * @WL_SHM_FORMAT_RGBX8888: (none)
+ * @WL_SHM_FORMAT_BGRX8888: (none)
+ * @WL_SHM_FORMAT_ABGR8888: (none)
+ * @WL_SHM_FORMAT_RGBA8888: (none)
+ * @WL_SHM_FORMAT_BGRA8888: (none)
+ * @WL_SHM_FORMAT_XRGB2101010: (none)
+ * @WL_SHM_FORMAT_XBGR2101010: (none)
+ * @WL_SHM_FORMAT_RGBX1010102: (none)
+ * @WL_SHM_FORMAT_BGRX1010102: (none)
+ * @WL_SHM_FORMAT_ARGB2101010: (none)
+ * @WL_SHM_FORMAT_ABGR2101010: (none)
+ * @WL_SHM_FORMAT_RGBA1010102: (none)
+ * @WL_SHM_FORMAT_BGRA1010102: (none)
+ * @WL_SHM_FORMAT_YUYV: (none)
+ * @WL_SHM_FORMAT_YVYU: (none)
+ * @WL_SHM_FORMAT_UYVY: (none)
+ * @WL_SHM_FORMAT_VYUY: (none)
+ * @WL_SHM_FORMAT_AYUV: (none)
+ * @WL_SHM_FORMAT_NV12: (none)
+ * @WL_SHM_FORMAT_NV21: (none)
+ * @WL_SHM_FORMAT_NV16: (none)
+ * @WL_SHM_FORMAT_NV61: (none)
+ * @WL_SHM_FORMAT_YUV410: (none)
+ * @WL_SHM_FORMAT_YVU410: (none)
+ * @WL_SHM_FORMAT_YUV411: (none)
+ * @WL_SHM_FORMAT_YVU411: (none)
+ * @WL_SHM_FORMAT_YUV420: (none)
+ * @WL_SHM_FORMAT_YVU420: (none)
+ * @WL_SHM_FORMAT_YUV422: (none)
+ * @WL_SHM_FORMAT_YVU422: (none)
+ * @WL_SHM_FORMAT_YUV444: (none)
+ * @WL_SHM_FORMAT_YVU444: (none)
  *
  * This describes the memory layout of an individual pixel.
+ *
+ * All renderers should support argb8888 and xrgb8888 but any other formats
+ * are optional and may not be supported by the particular renderer in use.
  */
 enum wl_shm_format {
 	WL_SHM_FORMAT_ARGB8888 = 0,
 	WL_SHM_FORMAT_XRGB8888 = 1,
+	WL_SHM_FORMAT_C8 = 0x20203843,
+	WL_SHM_FORMAT_RGB332 = 0x38424752,
+	WL_SHM_FORMAT_BGR233 = 0x38524742,
+	WL_SHM_FORMAT_XRGB4444 = 0x32315258,
+	WL_SHM_FORMAT_XBGR4444 = 0x32314258,
+	WL_SHM_FORMAT_RGBX4444 = 0x32315852,
+	WL_SHM_FORMAT_BGRX4444 = 0x32315842,
+	WL_SHM_FORMAT_ARGB4444 = 0x32315241,
+	WL_SHM_FORMAT_ABGR4444 = 0x32314241,
+	WL_SHM_FORMAT_RGBA4444 = 0x32314152,
+	WL_SHM_FORMAT_BGRA4444 = 0x32314142,
+	WL_SHM_FORMAT_XRGB1555 = 0x35315258,
+	WL_SHM_FORMAT_XBGR1555 = 0x35314258,
+	WL_SHM_FORMAT_RGBX5551 = 0x35315852,
+	WL_SHM_FORMAT_BGRX5551 = 0x35315842,
+	WL_SHM_FORMAT_ARGB1555 = 0x35315241,
+	WL_SHM_FORMAT_ABGR1555 = 0x35314241,
+	WL_SHM_FORMAT_RGBA5551 = 0x35314152,
+	WL_SHM_FORMAT_BGRA5551 = 0x35314142,
+	WL_SHM_FORMAT_RGB565 = 0x36314752,
+	WL_SHM_FORMAT_BGR565 = 0x36314742,
+	WL_SHM_FORMAT_RGB888 = 0x34324752,
+	WL_SHM_FORMAT_BGR888 = 0x34324742,
+	WL_SHM_FORMAT_XBGR8888 = 0x34324258,
+	WL_SHM_FORMAT_RGBX8888 = 0x34325852,
+	WL_SHM_FORMAT_BGRX8888 = 0x34325842,
+	WL_SHM_FORMAT_ABGR8888 = 0x34324241,
+	WL_SHM_FORMAT_RGBA8888 = 0x34324152,
+	WL_SHM_FORMAT_BGRA8888 = 0x34324142,
+	WL_SHM_FORMAT_XRGB2101010 = 0x30335258,
+	WL_SHM_FORMAT_XBGR2101010 = 0x30334258,
+	WL_SHM_FORMAT_RGBX1010102 = 0x30335852,
+	WL_SHM_FORMAT_BGRX1010102 = 0x30335842,
+	WL_SHM_FORMAT_ARGB2101010 = 0x30335241,
+	WL_SHM_FORMAT_ABGR2101010 = 0x30334241,
+	WL_SHM_FORMAT_RGBA1010102 = 0x30334152,
+	WL_SHM_FORMAT_BGRA1010102 = 0x30334142,
+	WL_SHM_FORMAT_YUYV = 0x56595559,
+	WL_SHM_FORMAT_YVYU = 0x55595659,
+	WL_SHM_FORMAT_UYVY = 0x59565955,
+	WL_SHM_FORMAT_VYUY = 0x59555956,
+	WL_SHM_FORMAT_AYUV = 0x56555941,
+	WL_SHM_FORMAT_NV12 = 0x3231564e,
+	WL_SHM_FORMAT_NV21 = 0x3132564e,
+	WL_SHM_FORMAT_NV16 = 0x3631564e,
+	WL_SHM_FORMAT_NV61 = 0x3136564e,
+	WL_SHM_FORMAT_YUV410 = 0x39565559,
+	WL_SHM_FORMAT_YVU410 = 0x39555659,
+	WL_SHM_FORMAT_YUV411 = 0x31315559,
+	WL_SHM_FORMAT_YVU411 = 0x31315659,
+	WL_SHM_FORMAT_YUV420 = 0x32315559,
+	WL_SHM_FORMAT_YVU420 = 0x32315659,
+	WL_SHM_FORMAT_YUV422 = 0x36315559,
+	WL_SHM_FORMAT_YVU422 = 0x36315659,
+	WL_SHM_FORMAT_YUV444 = 0x34325559,
+	WL_SHM_FORMAT_YVU444 = 0x34325659,
 };
 #endif /* WL_SHM_FORMAT_ENUM */
 
@@ -1442,6 +1562,7 @@ enum wl_pointer_axis {
 /**
  * wl_pointer - pointer input device
  * @set_cursor: set the pointer surface
+ * @release: release the pointer object
  *
  * The wl_pointer interface represents one or more input devices, such as
  * mice, which control the pointer location and pointer_focus of a seat.
@@ -1492,6 +1613,14 @@ struct wl_pointer_interface {
 			   struct wl_resource *surface,
 			   int32_t hotspot_x,
 			   int32_t hotspot_y);
+	/**
+	 * release - release the pointer object
+	 *
+	 * 
+	 * @since: 3
+	 */
+	void (*release)(struct wl_client *client,
+			struct wl_resource *resource);
 };
 
 #define WL_POINTER_ENTER	0
@@ -1562,6 +1691,24 @@ enum wl_keyboard_key_state {
 };
 #endif /* WL_KEYBOARD_KEY_STATE_ENUM */
 
+/**
+ * wl_keyboard - keyboard input device
+ * @release: release the keyboard object
+ *
+ * The wl_keyboard interface represents one or more keyboards associated
+ * with a seat.
+ */
+struct wl_keyboard_interface {
+	/**
+	 * release - release the keyboard object
+	 *
+	 * 
+	 * @since: 3
+	 */
+	void (*release)(struct wl_client *client,
+			struct wl_resource *resource);
+};
+
 #define WL_KEYBOARD_KEYMAP	0
 #define WL_KEYBOARD_ENTER	1
 #define WL_KEYBOARD_LEAVE	2
@@ -1598,6 +1745,30 @@ wl_keyboard_send_modifiers(struct wl_resource *resource_, uint32_t serial, uint3
 	wl_resource_post_event(resource_, WL_KEYBOARD_MODIFIERS, serial, mods_depressed, mods_latched, mods_locked, group);
 }
 
+/**
+ * wl_touch - touchscreen input device
+ * @release: release the touch object
+ *
+ * The wl_touch interface represents a touchscreen associated with a
+ * seat.
+ *
+ * Touch interactions can consist of one or more contacts. For each
+ * contact, a series of events is generated, starting with a down event,
+ * followed by zero or more motion events, and ending with an up event.
+ * Events relating to the same contact point can be identified by the ID of
+ * the sequence.
+ */
+struct wl_touch_interface {
+	/**
+	 * release - release the touch object
+	 *
+	 * 
+	 * @since: 3
+	 */
+	void (*release)(struct wl_client *client,
+			struct wl_resource *resource);
+};
+
 #define WL_TOUCH_DOWN	0
 #define WL_TOUCH_UP	1
 #define WL_TOUCH_MOTION	2
@@ -1789,6 +1960,247 @@ struct wl_region_interface {
 			 int32_t height);
 };
 
+#ifndef WL_SUBCOMPOSITOR_ERROR_ENUM
+#define WL_SUBCOMPOSITOR_ERROR_ENUM
+enum wl_subcompositor_error {
+	WL_SUBCOMPOSITOR_ERROR_BAD_SURFACE = 0,
+};
+#endif /* WL_SUBCOMPOSITOR_ERROR_ENUM */
+
+/**
+ * wl_subcompositor - sub-surface compositing
+ * @destroy: unbind from the subcompositor interface
+ * @get_subsurface: give a surface the role sub-surface
+ *
+ * The global interface exposing sub-surface compositing capabilities. A
+ * wl_surface, that has sub-surfaces associated, is called the parent
+ * surface. Sub-surfaces can be arbitrarily nested and create a tree of
+ * sub-surfaces.
+ *
+ * The root surface in a tree of sub-surfaces is the main surface. The main
+ * surface cannot be a sub-surface, because sub-surfaces must always have a
+ * parent.
+ *
+ * A main surface with its sub-surfaces forms a (compound) window. For
+ * window management purposes, this set of wl_surface objects is to be
+ * considered as a single window, and it should also behave as such.
+ *
+ * The aim of sub-surfaces is to offload some of the compositing work
+ * within a window from clients to the compositor. A prime example is a
+ * video player with decorations and video in separate wl_surface objects.
+ * This should allow the compositor to pass YUV video buffer processing to
+ * dedicated overlay hardware when possible.
+ */
+struct wl_subcompositor_interface {
+	/**
+	 * destroy - unbind from the subcompositor interface
+	 *
+	 * Informs the server that the client will not be using this
+	 * protocol object anymore. This does not affect any other objects,
+	 * wl_subsurface objects included.
+	 */
+	void (*destroy)(struct wl_client *client,
+			struct wl_resource *resource);
+	/**
+	 * get_subsurface - give a surface the role sub-surface
+	 * @id: the new subsurface object id
+	 * @surface: the surface to be turned into a sub-surface
+	 * @parent: the parent surface
+	 *
+	 * Create a sub-surface interface for the given surface, and
+	 * associate it with the given parent surface. This turns a plain
+	 * wl_surface into a sub-surface.
+	 *
+	 * The to-be sub-surface must not already have a dedicated purpose,
+	 * like any shell surface type, cursor image, drag icon, or
+	 * sub-surface. Otherwise a protocol error is raised.
+	 */
+	void (*get_subsurface)(struct wl_client *client,
+			       struct wl_resource *resource,
+			       uint32_t id,
+			       struct wl_resource *surface,
+			       struct wl_resource *parent);
+};
+
+#ifndef WL_SUBSURFACE_ERROR_ENUM
+#define WL_SUBSURFACE_ERROR_ENUM
+enum wl_subsurface_error {
+	WL_SUBSURFACE_ERROR_BAD_SURFACE = 0,
+};
+#endif /* WL_SUBSURFACE_ERROR_ENUM */
+
+/**
+ * wl_subsurface - sub-surface interface to a wl_surface
+ * @destroy: remove sub-surface interface
+ * @set_position: reposition the sub-surface
+ * @place_above: restack the sub-surface
+ * @place_below: restack the sub-surface
+ * @set_sync: set sub-surface to synchronized mode
+ * @set_desync: set sub-surface to desynchronized mode
+ *
+ * An additional interface to a wl_surface object, which has been made a
+ * sub-surface. A sub-surface has one parent surface. A sub-surface's size
+ * and position are not limited to that of the parent. Particularly, a
+ * sub-surface is not automatically clipped to its parent's area.
+ *
+ * A sub-surface becomes mapped, when a non-NULL wl_buffer is applied and
+ * the parent surface is mapped. The order of which one happens first is
+ * irrelevant. A sub-surface is hidden if the parent becomes hidden, or if
+ * a NULL wl_buffer is applied. These rules apply recursively through the
+ * tree of surfaces.
+ *
+ * The behaviour of wl_surface.commit request on a sub-surface depends on
+ * the sub-surface's mode. The possible modes are synchronized and
+ * desynchronized, see methods wl_subsurface.set_sync and
+ * wl_subsurface.set_desync. Synchronized mode caches the wl_surface state
+ * to be applied when the parent's state gets applied, and desynchronized
+ * mode applies the pending wl_surface state directly. A sub-surface is
+ * initially in the synchronized mode.
+ *
+ * Sub-surfaces have also other kind of state, which is managed by
+ * wl_subsurface requests, as opposed to wl_surface requests. This state
+ * includes the sub-surface position relative to the parent surface
+ * (wl_subsurface.set_position), and the stacking order of the parent and
+ * its sub-surfaces (wl_subsurface.place_above and .place_below). This
+ * state is applied when the parent surface's wl_surface state is applied,
+ * regardless of the sub-surface's mode. As the exception, set_sync and
+ * set_desync are effective immediately.
+ *
+ * The main surface can be thought to be always in desynchronized mode,
+ * since it does not have a parent in the sub-surfaces sense.
+ *
+ * Even if a sub-surface is in desynchronized mode, it will behave as in
+ * synchronized mode, if its parent surface behaves as in synchronized
+ * mode. This rule is applied recursively throughout the tree of surfaces.
+ * This means, that one can set a sub-surface into synchronized mode, and
+ * then assume that all its child and grand-child sub-surfaces are
+ * synchronized, too, without explicitly setting them.
+ *
+ * If the wl_surface associated with the wl_subsurface is destroyed, the
+ * wl_subsurface object becomes inert. Note, that destroying either object
+ * takes effect immediately. If you need to synchronize the removal of a
+ * sub-surface to the parent surface update, unmap the sub-surface first by
+ * attaching a NULL wl_buffer, update parent, and then destroy the
+ * sub-surface.
+ *
+ * If the parent wl_surface object is destroyed, the sub-surface is
+ * unmapped.
+ */
+struct wl_subsurface_interface {
+	/**
+	 * destroy - remove sub-surface interface
+	 *
+	 * The sub-surface interface is removed from the wl_surface
+	 * object that was turned into a sub-surface with
+	 * wl_subcompositor.get_subsurface request. The wl_surface's
+	 * association to the parent is deleted, and the wl_surface loses
+	 * its role as a sub-surface. The wl_surface is unmapped.
+	 */
+	void (*destroy)(struct wl_client *client,
+			struct wl_resource *resource);
+	/**
+	 * set_position - reposition the sub-surface
+	 * @x: coordinate in the parent surface
+	 * @y: coordinate in the parent surface
+	 *
+	 * This schedules a sub-surface position change. The sub-surface
+	 * will be moved so, that its origin (top-left corner pixel) will
+	 * be at the location x, y of the parent surface coordinate system.
+	 * The coordinates are not restricted to the parent surface area.
+	 * Negative values are allowed.
+	 *
+	 * The next wl_surface.commit on the parent surface will reset the
+	 * sub-surface's position to the scheduled coordinates.
+	 *
+	 * If more than one set_position request is invoked by the client
+	 * before the commit of the parent surface, the position of a new
+	 * request always replaces the scheduled position from any previous
+	 * request.
+	 *
+	 * The initial position is 0, 0.
+	 */
+	void (*set_position)(struct wl_client *client,
+			     struct wl_resource *resource,
+			     int32_t x,
+			     int32_t y);
+	/**
+	 * place_above - restack the sub-surface
+	 * @sibling: the reference surface
+	 *
+	 * This sub-surface is taken from the stack, and put back just
+	 * above the reference surface, changing the z-order of the
+	 * sub-surfaces. The reference surface must be one of the sibling
+	 * surfaces, or the parent surface. Using any other surface,
+	 * including this sub-surface, will cause a protocol error.
+	 *
+	 * The z-order is double-buffered. Requests are handled in order
+	 * and applied immediately to a pending state, then committed to
+	 * the active state on the next commit of the parent surface. See
+	 * wl_surface.commit and wl_subcompositor.get_subsurface.
+	 *
+	 * A new sub-surface is initially added as the top-most in the
+	 * stack of its siblings and parent.
+	 */
+	void (*place_above)(struct wl_client *client,
+			    struct wl_resource *resource,
+			    struct wl_resource *sibling);
+	/**
+	 * place_below - restack the sub-surface
+	 * @sibling: the reference surface
+	 *
+	 * The sub-surface is placed just below of the reference surface.
+	 * See wl_subsurface.place_above.
+	 */
+	void (*place_below)(struct wl_client *client,
+			    struct wl_resource *resource,
+			    struct wl_resource *sibling);
+	/**
+	 * set_sync - set sub-surface to synchronized mode
+	 *
+	 * Change the commit behaviour of the sub-surface to synchronized
+	 * mode, also described as the parent dependant mode.
+	 *
+	 * In synchronized mode, wl_surface.commit on a sub-surface will
+	 * accumulate the committed state in a cache, but the state will
+	 * not be applied and hence will not change the compositor output.
+	 * The cached state is applied to the sub-surface immediately after
+	 * the parent surface's state is applied. This ensures atomic
+	 * updates of the parent and all its synchronized sub-surfaces.
+	 * Applying the cached state will invalidate the cache, so further
+	 * parent surface commits do not (re-)apply old state.
+	 *
+	 * See wl_subsurface for the recursive effect of this mode.
+	 */
+	void (*set_sync)(struct wl_client *client,
+			 struct wl_resource *resource);
+	/**
+	 * set_desync - set sub-surface to desynchronized mode
+	 *
+	 * Change the commit behaviour of the sub-surface to
+	 * desynchronized mode, also described as independent or freely
+	 * running mode.
+	 *
+	 * In desynchronized mode, wl_surface.commit on a sub-surface will
+	 * apply the pending state directly, without caching, as happens
+	 * normally with a wl_surface. Calling wl_surface.commit on the
+	 * parent surface has no effect on the sub-surface's wl_surface
+	 * state. This mode allows a sub-surface to be updated on its own.
+	 *
+	 * If cached state exists when wl_surface.commit is called in
+	 * desynchronized mode, the pending state is added to the cached
+	 * state, and applied as whole. This invalidates the cache.
+	 *
+	 * Note: even if a sub-surface is set to desynchronized, a parent
+	 * sub-surface may override it to behave as synchronized. For
+	 * details, see wl_subsurface.
+	 *
+	 * If a surface's parent surface behaves as desynchronized, then
+	 * the cached state is applied on set_desync.
+	 */
+	void (*set_desync)(struct wl_client *client,
+			   struct wl_resource *resource);
+};
+
 #ifdef  __cplusplus
 }
 #endif
