@@ -92,6 +92,8 @@ struct wl_display {
 	struct wl_list client_list;
 
 	struct wl_signal destroy_signal;
+
+	struct wl_array additional_shm_formats;
 };
 
 struct wl_global {
@@ -112,21 +114,20 @@ struct wl_resource {
 	struct wl_client *client;
 	void *data;
 	int version;
+	wl_dispatcher_func_t dispatcher;
 };
 
 static int wl_debug = 0;
 
 WL_EXPORT void
-wl_resource_post_event(struct wl_resource *resource, uint32_t opcode, ...)
+wl_resource_post_event_array(struct wl_resource *resource, uint32_t opcode,
+			     union wl_argument *args)
 {
 	struct wl_closure *closure;
 	struct wl_object *object = &resource->object;
-	va_list ap;
 
-	va_start(ap, opcode);
-	closure = wl_closure_vmarshal(object, opcode, ap,
-				      &object->interface->events[opcode]);
-	va_end(ap);
+	closure = wl_closure_marshal(object, opcode, args,
+				     &object->interface->events[opcode]);
 
 	if (closure == NULL) {
 		resource->client->error = 1;
@@ -142,19 +143,32 @@ wl_resource_post_event(struct wl_resource *resource, uint32_t opcode, ...)
 	wl_closure_destroy(closure);
 }
 
-
 WL_EXPORT void
-wl_resource_queue_event(struct wl_resource *resource, uint32_t opcode, ...)
+wl_resource_post_event(struct wl_resource *resource, uint32_t opcode, ...)
 {
-	struct wl_closure *closure;
+	union wl_argument args[WL_CLOSURE_MAX_ARGS];
 	struct wl_object *object = &resource->object;
 	va_list ap;
 
 	va_start(ap, opcode);
-	closure = wl_closure_vmarshal(object, opcode, ap,
-				      &object->interface->events[opcode]);
+	wl_argument_from_va_list(object->interface->events[opcode].signature,
+				 args, WL_CLOSURE_MAX_ARGS, ap);
 	va_end(ap);
 
+	wl_resource_post_event_array(resource, opcode, args);
+}
+
+
+WL_EXPORT void
+wl_resource_queue_event_array(struct wl_resource *resource, uint32_t opcode,
+			      union wl_argument *args)
+{
+	struct wl_closure *closure;
+	struct wl_object *object = &resource->object;
+
+	closure = wl_closure_marshal(object, opcode, args,
+				     &object->interface->events[opcode]);
+
 	if (closure == NULL) {
 		resource->client->error = 1;
 		return;
@@ -169,6 +183,21 @@ wl_resource_queue_event(struct wl_resource *resource, uint32_t opcode, ...)
 	wl_closure_destroy(closure);
 }
 
+WL_EXPORT void
+wl_resource_queue_event(struct wl_resource *resource, uint32_t opcode, ...)
+{
+	union wl_argument args[WL_CLOSURE_MAX_ARGS];
+	struct wl_object *object = &resource->object;
+	va_list ap;
+
+	va_start(ap, opcode);
+	wl_argument_from_va_list(object->interface->events[opcode].signature,
+				 args, WL_CLOSURE_MAX_ARGS, ap);
+	va_end(ap);
+
+	wl_resource_queue_event_array(resource, opcode, args);
+}
+
 WL_EXPORT void
 wl_resource_post_error(struct wl_resource *resource,
 		       uint32_t code, const char *msg, ...)
@@ -298,8 +327,14 @@ wl_client_connection_data(int fd, uint32_t mask, void *data)
 		if (wl_debug)
 			wl_closure_print(closure, object, false);
 
-		wl_closure_invoke(closure, WL_CLOSURE_INVOKE_SERVER, object,
-				  opcode, client);
+		if ((resource_flags & WL_MAP_ENTRY_LEGACY) ||
+		    resource->dispatcher == NULL) {
+			wl_closure_invoke(closure, WL_CLOSURE_INVOKE_SERVER,
+					  object, opcode, client);
+		} else {
+			wl_closure_dispatch(closure, resource->dispatcher,
+					    object, opcode);
+		}
 
 		wl_closure_destroy(closure);
 
@@ -313,12 +348,30 @@ wl_client_connection_data(int fd, uint32_t mask, void *data)
 	return 1;
 }
 
+/** Flush pending events to the client
+ *
+ * \param client The client object
+ *
+ * Events sent to clients are queued in a buffer and written to the
+ * socket later - typically when the compositor has handled all
+ * requests and goes back to block in the event loop.  This function
+ * flushes all queued up events for a client immediately.
+ * 
+ * \memberof wl_client
+ */
 WL_EXPORT void
 wl_client_flush(struct wl_client *client)
 {
 	wl_connection_flush(client->connection);
 }
 
+/** Get the display object for the given client
+ *
+ * \param client The client object
+ * \return The display object the client is associated with.
+ * 
+ * \memberof wl_client
+ */
 WL_EXPORT struct wl_display *
 wl_client_get_display(struct wl_client *client)
 {
@@ -329,6 +382,28 @@ static void
 bind_display(struct wl_client *client,
 	     void *data, uint32_t version, uint32_t id);
 
+/** Create a client for the given file descriptor
+ *
+ * \param display The display object
+ * \param fd The file descriptor for the socket to the client
+ * \return The new client object or NULL on failure.
+ *
+ * Given a file descriptor corresponding to one end of a socket, this
+ * function will create a wl_client struct and add the new client to
+ * the compositors client list.  At that point, the client is
+ * initialized and ready to run, as if the client had connected to the
+ * servers listening socket.  When the client eventually sends
+ * requests to the compositor, the wl_client argument to the request
+ * handler will be the wl_client returned from this function.
+ *
+ * The other end of the socket can be passed to
+ * wl_display_connect_to_fd() on the client side or used with the
+ * WAYLAND_SOCKET environment variable on the client side.
+ *
+ * On failure this function sets errno accordingly and returns NULL.
+ * 
+ * \memberof wl_display
+ */
 WL_EXPORT struct wl_client *
 wl_client_create(struct wl_display *display, int fd)
 {
@@ -382,6 +457,25 @@ err_client:
 	return NULL;
 }
 
+/** Return Unix credentials for the client
+ *
+ * \param client The display object
+ * \param pid Returns the process ID
+ * \param uid Returns the user ID
+ * \param gid Returns the group ID
+ *
+ * This function returns the process ID, the user ID and the group ID
+ * for the given client.  The credentials come from getsockopt() with
+ * SO_PEERCRED, on the client socket fd.  All the pointers can be
+ * NULL, if the caller is not interested in a particular ID.
+ *
+ * Be aware that for clients that a compositor forks and execs and
+ * then connects using socketpair(), this function will return the
+ * credentials for the compositor.  The credentials for the socketpair
+ * are set at creation time in the compositor.
+ * 
+ * \memberof wl_client
+ */
 WL_EXPORT void
 wl_client_get_credentials(struct wl_client *client,
 			  pid_t *pid, uid_t *uid, gid_t *gid)
@@ -394,6 +488,17 @@ wl_client_get_credentials(struct wl_client *client,
 		*gid = client->ucred.gid;
 }
 
+/** Look up an object in the client name space
+ *
+ * \param client The client object
+ * \param id The object id
+ * \return The object or NULL if there is not object for the given ID
+ *
+ * This looks up an object in the client object name space by its
+ * object ID.  
+ * 
+ * \memberof wl_client
+ */
 WL_EXPORT struct wl_resource *
 wl_client_get_object(struct wl_client *client, uint32_t id)
 {
@@ -466,7 +571,7 @@ wl_resource_get_link(struct wl_resource *resource)
 WL_EXPORT struct wl_resource *
 wl_resource_from_link(struct wl_list *link)
 {
-	struct wl_resource *resource;
+	struct wl_resource *resource = NULL;
 
 	return wl_container_of(link, resource, link);
 }
@@ -585,11 +690,15 @@ registry_bind(struct wl_client *client,
 		if (global->name == name)
 			break;
 
-	if (&global->link == &display->global_list ||
-	    global->version < version)
+	if (&global->link == &display->global_list)
 		wl_resource_post_error(resource,
 				       WL_DISPLAY_ERROR_INVALID_OBJECT,
-				       "invalid global %d", name);
+				       "invalid global %s (%d)", interface, name);
+	else if (global->version < version)
+		wl_resource_post_error(resource,
+				       WL_DISPLAY_ERROR_INVALID_OBJECT,
+				       "invalid version for global %s (%d): have %d, wanted %d",
+				       interface, name, global->version, version);
 	else
 		global->bind(client, global->data, version, id);
 }
@@ -711,6 +820,8 @@ wl_display_create(void)
 	display->id = 1;
 	display->serial = 0;
 
+	wl_array_init(&display->additional_shm_formats);
+
 	if (!wl_global_create(display, &wl_display_interface, 1,
 			      display, bind_display)) {
 		wl_event_loop_destroy(display->loop);
@@ -742,6 +853,8 @@ wl_display_destroy(struct wl_display *display)
 	wl_list_for_each_safe(global, gnext, &display->global_list, link)
 		free(global);
 
+	wl_array_release(&display->additional_shm_formats);
+
 	free(display);
 }
 
@@ -794,12 +907,30 @@ wl_global_destroy(struct wl_global *global)
 	free(global);
 }
 
+/** Get the current serial number
+ *
+ * \param display The display object
+ *
+ * This function returns the most recent serial number, but does not
+ * increment it.
+ * 
+ * \memberof wl_display
+ */
 WL_EXPORT uint32_t
 wl_display_get_serial(struct wl_display *display)
 {
 	return display->serial;
 }
 
+/** Get the next serial number
+ *
+ * \param display The display object
+ *
+ * This function increments the display serial number and returns the
+ * new value.
+ * 
+ * \memberof wl_display
+ */
 WL_EXPORT uint32_t
 wl_display_next_serial(struct wl_display *display)
 {
@@ -1026,8 +1157,20 @@ wl_resource_set_implementation(struct wl_resource *resource,
 	resource->object.implementation = implementation;
 	resource->data = data;
 	resource->destroy = destroy;
+	resource->dispatcher = NULL;
 }
 
+WL_EXPORT void
+wl_resource_set_dispatcher(struct wl_resource *resource,
+			   wl_dispatcher_func_t dispatcher,
+			   const void *implementation,
+			   void *data, wl_resource_destroy_func_t destroy)
+{
+	resource->dispatcher = dispatcher;
+	resource->object.implementation = implementation;
+	resource->data = data;
+	resource->destroy = destroy;
+}
 
 WL_EXPORT struct wl_resource *
 wl_resource_create(struct wl_client *client,
@@ -1053,6 +1196,7 @@ wl_resource_create(struct wl_client *client,
 	resource->client = client;
 	resource->data = NULL;
 	resource->version = version;
+	resource->dispatcher = NULL;
 
 	if (wl_map_insert_at(&client->objects, 0, resource->object.id, resource) < 0) {
 		wl_resource_post_error(client->display_resource,
@@ -1168,3 +1312,55 @@ wl_display_remove_global(struct wl_display *display, struct wl_global *global)
 {
 	wl_global_destroy(global);
 }
+
+/** Add support for a wl_shm pixel format
+ *
+ * \param display The display object
+ * \param format The wl_shm pixel format to advertise
+ * \return A pointer to the wl_shm format that was added to the list
+ * or NULL if adding it to the list failed.
+ *
+ * Add the specified wl_shm format to the list of formats the wl_shm
+ * object advertises when a client binds to it.  Adding a format to
+ * the list means that clients will know that the compositor supports
+ * this format and may use it for creating wl_shm buffers.  The
+ * compositor must be able to handle the pixel format when a client
+ * requests it.
+ *
+ * The compositor by default supports WL_SHM_FORMAT_ARGB8888 and
+ * WL_SHM_FORMAT_XRGB8888.
+ *
+ * \memberof wl_display
+ */
+WL_EXPORT uint32_t *
+wl_display_add_shm_format(struct wl_display *display, uint32_t format)
+{
+	uint32_t *p = NULL;
+
+	p = wl_array_add(&display->additional_shm_formats, sizeof *p);
+
+	if (p != NULL)
+		*p = format;
+	return p;
+}
+
+/**
+ * Get list of additional wl_shm pixel formats
+ *
+ * \param display The display object
+ *
+ * This function returns the list of addition wl_shm pixel formats
+ * that the compositor supports.  WL_SHM_FORMAT_ARGB8888 and
+ * WL_SHM_FORMAT_XRGB8888 are always supported and not included in the
+ * array, but all formats added through wl_display_add_shm_format()
+ * will be in the array.
+ * 
+ * \sa wl_display_add_shm_format()
+ * 
+ * \memberof wl_display
+ */
+struct wl_array *
+wl_display_get_additional_shm_formats(struct wl_display *display)
+{
+	return &display->additional_shm_formats;
+}
